;Final Project - Optimal Trading Strategies of Agents in a Population of Firms: Soccer Transfer Markets
;Created by Kenny Salau
;11/17/2008
;This SLOW version of the football code shows long-run team distributions when initial skill of players on a given team is varied


turtles-own [salary tenure skill my_team teammates mates]
;*****Player Attributes*******************
;salary = benefits for a player 
;skill = skill level of a player
;tenure = number of season a player spends with one team
;teammates = total number of players on a given payers' team, including the player 
;mates = team members with same skill


patches-own [
              deltat_club  capital    alpha_m       
                  n_m      record     opponents
                strength   avg_skill  new_transfer
            ] 
;*********Firm Attributes**************
;n_m = Number of players at a given club
;alpha_m = revaluation rate, rate at which club manager decides to trade (or not) 
;capital = the amount of revenue a club gains in a season
;deltat_club = time till a club manager has to reevaluate his team
;record = Number of games own by a team in one season 
;opponents - the other teams a club competes against
;strength = the collective skill of all the players at a guven club
;avg_skill = The average skill of players at a given club
;new_transfer = the number of new transfers a club receives at the end of a transfer period

globals [avg_capital avg_tenure avg_salary avg_transfers min_deltat_club]    
;************Global Attributes***************
;avg_capital = Average capital of the entire football industry (whole lattice) 
;avg_tenure = Average Tenure length of players in the whole football industry
;avg_salary = Average salary of players in the whole football industry
;avg_transfer = average number of transfers occuring each season
;min_deltat_club = the club(s) with this attribute make the first decision on who to buy


;to setup
;  ca
;  ask n-of no-of-clubs patches [
;                                 set pcolor white                    ;Distinguishes clubs from patches
;                                 sprout no-of-players [establish]    ;populates clubs with an initial amount of players
;                                 organize                            ;gives an initial conditin of player attributes
;                               ]
;  ask turtles [set mates count turtles-here with [skill = [skill] of myself]]
;  ask patches with [pcolor = white] [set opponents other patches with [pcolor = white]]
;end

to setup
  ca
  let j 0
  while [j < no-of-clubs / 2]
    [ 
      ask (patch-set patch (2 * j + 2) -40 patch (-2 * j - 2) -40)
        [
          set pcolor white                                              ;Distinguishes clubs from patches
          sprout no-of-players [                                        ;populates clubs with an initial amount of players
                                 establish                              ;gives an initial condition of player attributes
                                 move-to patch-at 0 1
                                 while [count other turtles-here != 0] [move-to patch-at 0 1]
                               ]                                        
        ]
      set j j + 1
    ] 
  ask patches with [pcolor = white] [                                               ;this piece of code establishes what kind of teams will be present in the league (great, mediocre, etc.)
                                      let k random 100
                                      ask turtles with [xcor = [pxcor] of myself] 
                                        [
                                          ifelse k <= poor-team-density
                                            [
                                              poor-team
                                            ]
                                            [ifelse k <= poor-team-density + mediocre-team-density
                                              [mediocre-team]
                                              [great-team]
                                            ]
                                         ]
                                      set opponents other patches with [pcolor = white]
                                      organize
                                    ]
  ask turtles [
                set my_team one-of patches with [pcolor = white and pxcor = [xcor] of myself]
                set teammates count turtles with [xcor = [pxcor] of myself]
                set mates count turtles with [xcor = [pxcor] of myself and skill = [skill] of myself]
              ]
end

to go
  ask turtles [reestablish_pre_season]
  ask patches with [pcolor = white] [reorganize_pre_season]
  simulate-season
  ask turtles [reestablish_post_season]
  ask patches with [pcolor = white] [reorganize_post_season]
  let i 0
  while [i < no-of-clubs]
    [
      set min_deltat_club min [deltat_club] of patches with [pcolor = white]
      negotiate
      set i i + 1
    ]
  ask turtles [reestablish_post_negotiation]
  ask patches with [pcolor = white] [reorganize_post_negotiation]
  set avg_transfers sum [new_transfer] of patches with [pcolor = white] / count patches with [pcolor = white]
  set avg_tenure sum [tenure] of turtles / count turtles
  tick
end

to establish              ;This spawns the initial conditions for player attributes
  set salary 0 
  set tenure 0
end

to great-team
 let r random 100
 ifelse r <= 10
   [
     set skill 1
     set color green
   ]
   [ifelse r <= 40
     [
       set skill 2
       set color blue
     ]
     [
       set skill 3
       set color yellow
     ]
   ]
end

to mediocre-team
 let g random 100
 ifelse g <= 20
   [
     set skill 1
     set color green
   ]
   [ifelse g <= 80
     [
       set skill 2
       set color blue
     ]
     [
       set skill 3
       set color yellow
     ]
   ]
end
to poor-team
 let b random 100
 ifelse b <= 70
   [
     set skill 1
     set color green
   ]
   [ifelse b <= 90
     [
       set skill 2
       set color blue
     ]
     [
       set skill 3
       set color yellow
     ]
   ]
end
to reestablish_pre_season              ;players update their attributes at the start of a season
  set salary 0
end

to reestablish_post_season              ;players update their attributes at the end of a season
  set salary (1 / 2) * ([record] of my_team) * (skill / 6) / (mates)
  set tenure tenure + 1
end

to reestablish_post_negotiation              ;players update their attributes at the end of the transfer period
  set tenure tenure
  set my_team one-of patches with [pcolor = white and pxcor = [xcor] of myself]
  set teammates count turtles with [xcor = [pxcor] of myself]
  set mates count turtles with [xcor = [pxcor] of myself and skill = [skill] of myself]
end

to organize                  ;This spawns the initial conditions for club attributes
  set n_m count turtles with [xcor = [pxcor] of myself]
  set record 0
  set capital (1 / 2) * record
  set alpha_m n_m / alpha
  set strength sum [skill] of turtles with [xcor = [pxcor] of myself]
  set avg_skill precision (strength / n_m) 2
  set new_transfer 0
end

to reorganize_pre_season                  ;clubs update their attributes at the start of a season
  set record 0
  set capital 0
  set opponents other patches with [pcolor = white]
  set deltat_club (0 - ln(random-float 1)) * alpha_m 
end

to reorganize_post_season                  ;clubs update their attributes at the end of a season
  set capital (1 / 2) * record
end

to reorganize_post_negotiation                  ;clubs update their attributes at the end of the transfer period
  set n_m count turtles with [xcor = [pxcor] of myself]
  set capital (1 / 2) * record
  set alpha_m n_m / alpha
  set strength sum [skill] of turtles with [xcor = [pxcor] of myself]
  set avg_skill precision (strength / n_m) 2
end

to simulate-season            ;This makes the clubs play games with each other, a given club plays two games with each opponent (home & away)
  ask patches with [pcolor = white] 
    [
      ask other patches with [pcolor = white]
           [
              ifelse random-float 1 < ([strength] of myself) / (strength + [strength] of myself)    ;a give club wins a game with a probability determined by the strength of their squad
                [set [record] of myself [record] of myself + 1]
                [set record record + 1]
           ]
     ]
end

to negotiate
  ask patches with [pcolor = white and deltat_club = min_deltat_club]
    [
      ask n-of prospects turtles with [xcor != [pxcor] of myself and teammates > 14]   ;prospects are the number of players a club is allowed to negotiate with
        [
          if random-float 1 < (skill / [avg_skill] of myself) ^ beta    ;beta affects the probability that a team wants a players whose skill does not match or exceed the average          
            [
              if ([capital] of myself) / prospects > rho * (salary)     ;rho affects the amount profit the player is expecting, personally, greater than 1 is good 
                [
                  if ((1 / 2) * ([record] of myself) * (skill / 6)) / ([n_m] of myself + 1) > salary
                    [
                      show (word self "of" my_team "moved to" myself)
                      move-to myself
                      move-to patch-at 0 1
                      while [count other turtles-here != 0] [move-to patch-at 0 1]
                      set tenure 0
                      set [new_transfer] of myself [new_transfer] of myself + 1
                    ]
                ]
            ]
        ]
      set deltat_club 100000000000000000000000000          ;this is a primitive way of ensuring that after a manager makes a post-season trade decision it cannot make another one
    ]
end




;to simulate-season
;  let i 1
;  ask patches with [pcolor = white] 
;    [
;      while [i < no-of-clubs]
;        [
;          ask one-of opponents
;           [
;              ifelse random-float 1 < ([strength] of self) / ([strength] of self + [strength] of myself)
;                [set record record + 1]
;                [set [record] of myself [record] of myself + 1]
;              set opponents remove self opponents
;           ]
;          set i (i + 1)
;        ]
;     ]
;end

@#$#@#$#@
GRAPHICS-WINDOW
221
10
723
794
26
40
9.3
1
10
1
1
1
0
1
1
1
-26
26
-40
40
0
0
1
ticks

CC-WINDOW
5
808
1105
903
Command Center
0

BUTTON
18
24
85
58
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
15
164
188
197
no-of-clubs
no-of-clubs
0
20
20
1
1
NIL
HORIZONTAL

SLIDER
15
210
188
243
no-of-players
no-of-players
0
30
22
1
1
NIL
HORIZONTAL

SLIDER
742
39
915
72
alpha
alpha
0
10
1
0.5
1
NIL
HORIZONTAL

BUTTON
16
71
173
104
simulate one season
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
13
120
187
153
simulate many seasons
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
742
85
914
118
beta
beta
0
5
1
1
1
NIL
HORIZONTAL

SLIDER
741
135
913
168
prospects
prospects
0
10
5
1
1
NIL
HORIZONTAL

SLIDER
728
180
912
213
poor-team-density
poor-team-density
0
100
25
1
1
%
HORIZONTAL

SLIDER
727
225
938
258
mediocre-team-density
mediocre-team-density
0
100
55
1
1
%
HORIZONTAL

TEXTBOX
945
179
1095
263
For the seup to make sense, make sure poor and mediocre team densities dont exceed 100, the 'great' team density is intuitive
11
0.0
1

MONITOR
36
255
134
300
NIL
avg_transfers
17
1
11

MONITOR
38
315
120
360
NIL
avg_tenure
17
1
11

SLIDER
924
39
1096
72
rho
rho
0
10
1
1
1
NIL
HORIZONTAL

@#$#@#$#@
WHAT IS IT?
-----------
This section could give a general understanding of what the model is trying to show or explain.


HOW IT WORKS
------------
This section could explain what rules the agents use to create the overall behavior of the model.


HOW TO USE IT
-------------
This section could explain how to use the model, including a description of each of the items in the interface tab.


THINGS TO NOTICE
----------------
This section could give some ideas of things for the user to notice while running the model.


THINGS TO TRY
-------------
This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.


EXTENDING THE MODEL
-------------------
This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.


NETLOGO FEATURES
----------------
This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.


RELATED MODELS
--------------
This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.


CREDITS AND REFERENCES
----------------------
This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.0.3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="20"/>
    <metric>[avg_skill] of patches with [pcolor = white]</metric>
    <metric>avg_tenure</metric>
    <enumeratedValueSet variable="no-of-players">
      <value value="22"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-of-clubs">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="poor-team-density">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="prospects">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="beta">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="alpha">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mediocre-team-density">
      <value value="55"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
